#!/bin/bash

AWK="/usr/bin/awk"
ECHO="/bin/echo"
GREP="/usr/bin/grep"
EGREP="/usr/bin/egrep"
ID="/usr/xpg4/bin/id"
INETADM="/usr/sbin/inetadm"
LS="/bin/ls"
PKGCHK="/usr/sbin/pkgchk"
PKGINFO="/usr/bin/pkginfo"
SED="/usr/bin/sed"
SVCADM="/usr/sbin/svcadm"
SVCCFG="/usr/sbin/svccfg"
SVCS="/usr/bin/svcs"
UNAME="/bin/uname"
WC="/bin/wc"

OS="unknown"
OS_VERSION="unknown"

SUCCESS=0
ERROR=1

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function get_owner {
   ${LS} -ld "$1" | ${AWK} '{print $3}'
}

function get_group {
   ${LS} -ld "$1" | ${AWK} '{print $4}'
}

function get_user_perms {
   ${LS} -ld "$1" | ${AWK} '{print substr($1,2,3)}'
}

function get_group_perms {
   ${LS} -ld "$1" | ${AWK} '{print substr($1,5,3)}'
}

function get_other_perms {
   ${LS} -ld "$1" | ${AWK} '{print substr($1,8,3)}'
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function check_inetd_tracing {
   print_banner "Checking inetd Tracing"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${INETADM} -p | ${GREP} "^tcp_trace=TRUE" >/dev/null 2>&1
          if [ "$?" -ne "0" ]; then
             print_warn "inetd tracing is not enabled"
             print_info "Consider turning this on with the following command:"
             print_info "${INETADM} -M tcp_trace=true"
             print_info "or, for individual services"
             print_info "${INETADM} -m <svcname> tcp_trace=true"
          else
             print_ok "inetd tracing is enabled"
          fi
          ;;
   esac
}

function check_ftp_logging {
   print_banner "Checking FTP Logging"
   print_divider
   case ${OS_VERSION} in
      "10")
          ${INETADM} | ${EGREP} "(on|off)line.*svc:/network/ftp" >/dev/null 2>&1
          if [ "$?" -eq "0" ]; then 
             EXEC_STRING=$( ${INETADM} -l svc:/network/ftp | ${GREP} "^[ ][ ]*exec" )
             EXEC_STRING=$( ${ECHO} "${EXEC_STRING}" | ${SED} 's/^.*exec=\"\(.*\)\"$/\1/' )
             # assumes arguments in a specific order - oh well...
             if [ "${EXEC_STRING}" != "/usr/sbin/in.ftpd -a -l -d" ]; then
                print_warn "Additional logging for FTP daemon is not configured"
                print_info "Execute the following command to enable connection logging and debugging:"
                print_info "${INETADM} -m svc:/network/ftp exec=\"/usr/sbin/in.ftpd -a -l -d\"" 
             else
                print_ok "Additional logging for FTP daemon is configured correctly"
             fi
          else
             print_ok "FTP disabled, not installed, or broken"
          fi
          ;;
   esac
}

function check_connection_logging {
   print_banner "Checking Connection Logging"
   print_divider
   case ${OS_VERSION} in
     "10")
         print_info "Checking whether we are logging daemon.debug to a connection log"
         ${GREP} "^daemon.debug.*/var/log/connlog" /etc/syslog.conf >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "Not logging daemon.debug to /var/log/connlog"
            print_info "It is recommended that you execute the following commands"
            print_info "to instate a connection log:"
            print_info "echo -e \"daemon.debug\\\t\\\t\\\t/var/log/connlog\\\n\" >> /etc/syslog.conf"
            print_info "touch /var/log/connlog"
            print_info "chown root:root /var/log/connlog"
            print_info "chmod 600 /var/log/connlog"
            print_info "logadm -w connlog -C 13 -a 'pkill -HUP syslogd' /var/log/connlog"
            # This should really be svcadm refresh system-log... but HUP still works
            # print_info "logadm -w connlog -C 13 -a 'pkill -HUP syslogd' /var/log/connlog"
         else
            print_ok "daemon.debug logging to /var/log/connlog enabled"
         fi
         ;;
   esac
}


function check_syslog_auth_logging {
   print_banner "Checking syslog AUTH Logging"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^auth.info.*/var/log/authlog" /etc/syslog.conf >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "Not logging auth.info to /var/log/authlog"
            print_info "It is recommended that you execute the following commands"
            print_info "to capture messages sent to the syslog AUTH facility:"
            print_info "echo -e \"auth.info\\\t\\\t\\\t/var/log/authlog\\\n\" >> /etc/syslog.conf"
            print_info "touch /var/log/authlog"
            print_info "chown root:root /var/log/authlog"
            print_info "chmod 600 /var/log/authlog"
            print_info "logadm -w authlog -C 13 -a 'pkill -HUP syslogd' /var/log/authlog"
            
         else
            print "auth.info logging to /var/log/authlog enabled"
         fi
         ;;
   esac
}

function check_failed_login_logging {
   print_banner "Checking logging of failed login attempts"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^SYSLOG_FAILED_LOGINS=0" /etc/default/login >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "SYSLOG_FAILED_LOGINS not set to 0 in /etc/default/login"
            print_info "It is recommended that you execute the following commands"
            print_info "to log all failed logins to /var/adm/loginlog:"
            print_info "touch /var/adm/loginlog"
            print_info "chown root:sys /var/adm/loginlog"
            print_info "chmod 600 /var/adm/loginlog"
            print_info "cd /etc/default"
            print_info "sed 's/^[#]\{0,1\}SYSLOG_FAILED_LOGINS=.*$/SYSLOG_FAILED_LOGINS=0/' login > login.new"
            print_info "mv login.new login"
            print_info "pkgchk -f -n -p /etc/default/login"
            print_info "logadm -w connlog -C 13 /var/adm/loginlog"
         else
            print_ok "SYSLOG_FAILED_LOGINS configured to log all failed login attempts"
            if [ -e "/var/adm/loginlog" ]; then
               print_ok "/var/adm/loginlog exists"
            else
               print_warn "/var/adm/loginlog does not exist"
            fi
         fi
         ;;
   esac
}

function check_cron_logging {
   print_banner "Checking cron logging"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^CRONLOG=YES" /etc/default/cron >/dev/null 2>&1
         if [ "$?" -eq "0" ]; then 
            print_warn "cron logging not enabled"
            print_info "It is recommended that you execute the following commands"
            print_info "to enable cron logging:"
            print_info "cd /etc/default"
            print_info "sed 's/^[#]\{0,1\}CRONLOG=.*$/CRONLOG=YES/' cron > cron.new"
            print_info "mv cron.new cron"
            print_info "pkgchk -f -n -p /etc/default/cron"
            print_info "svcadm refresh svc:/system/cron:default"
         else
            print_ok "cron logging is enabled"
         fi 
         ;;
   esac
}

function check_system_accounting {
   print_banner "Checking System Accounting"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${SVCS} -a | ${GREP} "svc:/system/sar:default" >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "SAR not registered with SMF... checking for package installation" 
            PACKAGES="SUNWaccu SUNWaccr"
            for PACKAGE in ${PACKAGES}; do 
               ${PKGINFO} ${PACKAGE} >/dev/null 2>&1 
               if [ "$?" -eq "0" ]; then
                  print_ok "Package ${PACKAGE} is installed"
               else
                  print_warn "Package ${PACKAGE} is not installed"
               fi
             done
         else
            SAR_STATE=$( ${SVCS} svc:/system/sar:default | ${SED} -n '$p' | ${AWK} '{print $1}' )
            if [ "${SAR_STATE}" != "online" ]; then
               print_warn "Service svc:/system/sar:default is [${SAR_STATE}]"
            else
               print_ok "Service svc:/system/sar:default is online"
            fi
         fi 
         if [ ! -e "/var/spool/cron/crontabs/sys" ]; then
            print_warn "No crontab exists for user \"sys\""
            print_info "Use \"crontab -e sys\" to create it"
         fi
         CRON_ERRORS=0
         ${GREP} "^0,20,40 \* \* \* \* /usr/lib/sa/sa1$" /var/spool/cron/crontabs/sys >/dev/null 2>&1 || (( CRON_ERRORS = CRON_ERRORS + 1 ))
         ${GREP} "^45 23 \* \* \* /usr/lib/sa/sa2 -s 0:00 -e 23:59 -i 1200 -A$" /var/spool/cron/crontabs/sys >/dev/null 2>&1 || (( CRON_ERRORS = CRON_ERRORS + 1 ))
         if [ "${CRON_ERRORS}" -gt "0" ]; then
            print_warn "One or more System Accounting entries in crontab for user \"sys\""
            print_warn "not present or incorrect. Ensure the following entries are present:"
            print_info "0,20,40 * * * * /usr/lib/sa/sa1"
            print_info "45 23 * * * /usr/lib/sa/sa2 -s 0:00 -e 23:59 -i 1200 -A"
         fi
         ;;
   esac
}

function check_log_file_permissions {
   print_banner "Checking System Log File Permissions"
   print_divider
   case ${OS_VERSION} in
     "10")
         print_info "Will now check /var/log/syslog, /var/log/authlog, /var/adm/utmpx, /var/adm/wtmpx"
         print_info "with pkgchk. Review output (if any) for inconsistencies."
         LOGFILES="/var/log/syslog /var/log/authlog /var/adm/utmpx /var/adm/wtmpx"
         ERROR_COUNT=0
         for LOGFILE in ${LOGFILES}; do
            if [ -e "${LOGFILE}" ]; then
               ${PKGCHK} -p ${LOGFILE}
               if [ "$?" -ne "0" ]; then 
                  (( ERROR_COUNT = ERROR_COUNT + 1 ))
               fi
            fi
         done
         if [ "${ERROR_COUNT}" -eq "0" ]; then
            print_ok "No errors found"
         else
            print_warn "Errors found whilst checking logfiles with pkgchk"
         fi
         # Logs that should be chown root:sys
         LOGFILES="/var/adm/loginlog"
         for LOGFILE in ${LOGFILES}; do
            if [ -e "${LOGFILE}" ]; then
               LOGFILE_ONWER=$( get_owner ${LOGFILE} )
               LOGFILE_GROUP=$( get_group ${LOGFILE} )
               if [ "${LOGFILE_ONWER}" != "root" ]; then
                  print_warn "${LOGFILE} not owned by \"root\""
                  print_info "Execute the following command to fix this problem:"
                  print_info "chown root ${LOGFILE}"
               fi
               if [ "${LOGFILE_GROUP}" != "sys" ]; then
                  print_warn "${LOGFILE} group not \"sys\""
                  print_info "Execute the following command to fix this problem:"
                  print_info "chgrp sys ${LOGFILE}"
               fi
            fi
         done
         # Logs that should be chown root:root
         LOGFILES="/var/cron/log /var/adm/messages /var/log/connlog"
         for LOGFILE in ${LOGFILES}; do
            if [ -e "${LOGFILE}" ]; then
               LOGFILE_ONWER=$( get_owner ${LOGFILE} )
               LOGFILE_GROUP=$( get_group ${LOGFILE} )
               if [ "${LOGFILE_ONWER}" != "root" ]; then
                  print_warn "${LOGFILE} not owned by \"root\""
                  print_info "Execute the following command to fix this problem:"
                  print_info "chown root ${LOGFILE}"
               fi
               if [ "${LOGFILE_GROUP}" != "root" ]; then
                  print_warn "${LOGFILE} group not \"root\""
                  print_info "Execute the following command to fix this problem:"
                  print_info "chgrp root ${LOGFILE}"
               fi
            fi
         done
         # Now time to check permissions
         # /var/adm/messages should be chmod go-wx
         LOGFILES="/var/adm/messages"
         for LOGFILE in ${LOGFILES}; do
            if [ -e "${LOGFILE}" ]; then
               GROUP_PERMS=$( get_group_perms ${LOGFILE} )
               OTHER_PERMS=$( get_other_perms ${LOGFILE} )
               if [ "${GROUP_PERMS}" != "r--" ]; then
                  print_warn "${LOGFILE} group permissions not r--"
                  print_info "Execute the following command to fix this problem:"
                  print_info "chmod g=r ${LOGFILE}"
               fi
               if [ "${OTHER_PERMS}" != "r--" ]; then
                  print_warn "${LOGFILE} other permissions not r--"
                  print_info "Execute the following command to fix this problem:"
                  print_info "chmod o= ${LOGFILE}"
               fi
            fi
         done
         # Logfiles that should be chmod go-rwx
         LOGFILES="/var/adm/loginlog /var/cron/log /var/log/connlog"
         for LOGFILE in ${LOGFILES}; do
            if [ -e "${LOGFILE}" ]; then
               GROUP_PERMS=$( get_group_perms ${LOGFILE} )
               OTHER_PERMS=$( get_other_perms ${LOGFILE} )
               if [ "${GROUP_PERMS}" != "---" ]; then
                  print_warn "${LOGFILE} group permissions not ---"
                  print_info "Execute the following command to fix this problem:"
                  print_info "chmod g= ${LOGFILE}"
               fi
               if [ "${OTHER_PERMS}" != "---" ]; then
                  print_warn "${LOGFILE} other permissions not ---"
                  print_info "Execute the following command to fix this problem:"
                  print_info "chmod o= ${LOGFILE}"
               fi
            fi             
         done
         # Logfiles that should be chown sys:sys AND chmod go-wx
         LOGFILES="/var/adm/sa/*"
         for LOGFILE in ${LOGFILES}; do
            if [ -e "${LOGFILE}" ]; then
               LOGFILE_ONWER=$( get_owner ${LOGFILE} )
               LOGFILE_GROUP=$( get_group ${LOGFILE} )
               GROUP_PERMS=$( get_group_perms ${LOGFILE} )
               OTHER_PERMS=$( get_other_perms ${LOGFILE} )
               if [ "${LOGFILE_ONWER}" != "sys" ]; then
                  print_warn "${LOGFILE} not owned by \"sys\""
                  print_info "Execute the following command to fix this problem:"
                  print_info "chown sys ${LOGFILE}"
               fi
               if [ "${LOGFILE_GROUP}" != "sys" ]; then
                  print_warn "${LOGFILE} group not \"sys\""
                  print_info "Execute the following command to fix this problem:"
                  print_info "chgrp sys ${LOGFILE}"
               fi
               if [ "${GROUP_PERMS}" != "r--" ]; then
                  print_warn "${LOGFILE} group permissions not r--"
                  print_info "Execute the following command to fix this problem:"
                  print_info "chmod g=r ${LOGFILE}"
               fi
               if [ "${OTHER_PERMS}" != "r--" ]; then
                  print_warn "${LOGFILE} other permissions not r--"
                  print_info "Execute the following command to fix this problem:"
                  print_info "chmod o=r ${LOGFILE}"
               fi
            fi
         done
         ;;
   esac
}

check_os
check_user
print_divider
check_inetd_tracing
print_divider
check_ftp_logging
print_divider
check_connection_logging
print_divider
check_syslog_auth_logging
print_divider
check_failed_login_logging 
print_divider
check_cron_logging
print_divider
check_system_accounting
print_divider
check_log_file_permissions
print_divider

exit 0
